/*
 * Copyright (C) 2003-2008 eXo Platform SAS.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see<http://www.gnu.org/licenses/>.
 */
package org.exoplatform.services.jcr.ext.replication.test;

import java.io.IOException;
import java.io.InputStream;

import javax.jcr.Credentials;
import javax.jcr.Node;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.core.CredentialsImpl;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SAS.
 * 
 * @author <a href="mailto:alex.reshetnyak@exoplatform.com.ua">Alex Reshetnyak</a>
 * @version $Id: BaseReplicationTestCase.java 21165 2008-10-10 08:02:05Z rainf0x $
 */

public abstract class BaseReplicationTestCase {
  protected static Log       log         = ExoLogger.getLogger("ext.AbstractReplicationTestCase");

  protected static final int BUFFER_SIZE = 1024;

  public static final int    DIVIDER     = 255;

  public static final double ONE_SECONDS = 1000;

  protected Session          session;

  protected Node             rootNode;

  private Credentials        credentials;

  protected Repository       repository;

  public BaseReplicationTestCase(RepositoryService repositoryService,
                                 String reposytoryName,
                                 String workspaceName,
                                 String userName,
                                 String password) {
    try {
      credentials = new CredentialsImpl(userName, password.toCharArray());

      repository = repositoryService.getRepository(reposytoryName);

      session = repository.login(credentials, workspaceName);

      rootNode = session.getRootNode();

    } catch (RepositoryException e) {
      log.error("Can't start BaseReplicationTestCase", e);
    } catch (RepositoryConfigurationException e) {
      log.error("Can't start BaseReplicationTestCase", e);
    }
  }

  protected Node addNodePath(String repoPath) throws RepositoryException {
    Node resultNode = rootNode;
    String[] sArray = repoPath.split("[::]");

    for (String nodeName : sArray)
      if (resultNode.hasNode(nodeName))
        resultNode = resultNode.getNode(nodeName);
      else
        resultNode = resultNode.addNode(nodeName, "nt:unstructured");

    return resultNode;
  }

  protected String getNormalizePath(String repoPath) {
    return repoPath.replaceAll("[:][:]", "/");
  }

  protected void compareStream(InputStream etalon, InputStream data) throws Exception {
    compareStream(etalon, data, 0, 0, -1);
  }

  /**
   * Compare etalon stream with data stream begining from the offset in etalon and position in data.
   * Length bytes will be readed and compared. if length is lower 0 then compare streams till one of
   * them will be read.
   * 
   * @param etalon
   * @param data
   * @param etalonPos
   * @param dataPos
   * @param length
   * @throws Exception
   */
  protected void compareStream(InputStream etalon,
                               InputStream data,
                               long etalonPos,
                               long dataPos,
                               long length) throws Exception {

    int dindex = 0;

    byte[] ebuff = new byte[BUFFER_SIZE];
    int eread = 0;

    while ((eread = etalon.read(ebuff)) > 0) {

      byte[] dbuff = new byte[eread];
      int erindex = 0;
      while (erindex < eread) {
        int dread = -1;
        try {
          dread = data.read(dbuff);
        } catch (IOException e) {
          throw new Exception("Streams is not equals by length or data stream is unreadable. Cause: "
              + e.getMessage());
        }

        if (dread == -1)
          throw new Exception("Streams is not equals by length. Data end-of-stream reached at position "
              + dindex);

        for (int i = 0; i < dread; i++) {
          byte eb = ebuff[i];
          byte db = dbuff[i];
          if (eb != db)
            throw new Exception("Streams is not equals. Wrong byte stored at position " + dindex
                + " of data stream. Expected 0x" + Integer.toHexString(eb) + " '"
                + new String(new byte[] { eb }) + "' but found 0x" + Integer.toHexString(db) + " '"
                + new String(new byte[] { db }) + "'");

          erindex++;
          dindex++;
          if (length > 0 && dindex >= length)
            return; // tested length reached
        }

        if (dread < eread)
          dbuff = new byte[eread - dread];
      }
    }

    if (data.available() > 0)
      throw new Exception("Streams is not equals by length. Data stream contains more data. Were read "
          + dindex);
  }
}
